/* -*- c++ -*- */
/**************************************************************************
 Copyright (c) 2000-2001, Tony Garnock-Jones
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:

     * Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.

     * Redistributions in binary form must reproduce the above
       copyright notice, this list of conditions and the following
       disclaimer in the documentation and/or other materials provided
       with the distribution.

     * Neither the names of the copyright holders nor the names of this
       software's contributors may be used to endorse or promote
       products derived from this software without specific prior
       written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

#ifndef MPS_Interface_H
#define MPS_Interface_H

namespace MPS {

class InterfaceServer;

/**
 * Base class which all MPS Interfaces (implementations and proxies)
 * must inherit from. */
class Interface: public Referenceable {
private:
  /// Server that is presenting bound addresses for this interface
  InterfaceServer *interfaceServer;

  /// If we are a proxy, this holds the bound address of the server we are proxying for
  string remoteAddress;

  friend class InterfaceServer;	// allow access to _setInterfaceServer.

  /// Called by InterfaceServer::InterfaceServer. Yes, cleaner solutions do exist.
  void _setInterfaceServer(InterfaceServer *s) {
    if (interfaceServer)
      throw MPSException("_setInterfaceServer called when interfaceServer already non-zero");
    interfaceServer = s;
  }

protected:
  /// Called by auto-generated subclasses of Proxy, in their constructors. See gencpp.cc.
  void _setRemoteAddress(string const &addr) {
    remoteAddress = addr;
  }

public:
  Interface()
    : interfaceServer(0),
      remoteAddress()
  {}

  virtual ~Interface() {}

  /**
   * Retrieve the server object that is providing access to this
   * instance of Interface, or 0 if this Interface isn't (yet) managed
   * by an InterfaceServer. */
  InterfaceServer *_getInterfaceServer() const { return interfaceServer; }

  /**
   * Returns the address of the remote object that we are using, or
   * the empty-string if we are not a proxy for some remote object
   * (ie we really do implement this interface locally). */
  string const &_getRemoteAddress() const { return remoteAddress; }
};

}

#endif
